# -*- coding: utf-8 -*-

import logging

from tools import trace, timer

from View.ASubWindow import ASubMainWindow
from View.ListedSubWindow import ListedSubWindow

from PyQt5.QtGui import (
    QKeySequence,
)

from PyQt5.QtCore import (
    Qt, QVariant, QAbstractTableModel,
    QCoreApplication, QModelIndex, pyqtSlot,
    QRect,
)

from PyQt5.QtWidgets import (
    QDialogButtonBox, QPushButton, QLineEdit,
    QFileDialog, QTableView, QAbstractItemView,
    QUndoStack, QShortcut, QAction, QItemDelegate,
    QComboBox, QVBoxLayout, QHeaderView, QTabWidget,
)

from View.Frictions.UndoCommand import (
    PasteCommand, DuplicateCommand,
)

from View.Frictions.Table import (
    TableModel, ComboBoxDelegate
)

from View.Plot.MplCanvas import MplCanvas
from View.Geometry.PlotKPZ import PlotKPZ
from View.Frictions.PlotStricklers import PlotStricklers
from View.Frictions.translate import *

from View.Stricklers.Window import StricklersWindow

_translate = QCoreApplication.translate

logger = logging.getLogger()

class FrictionsWindow(ASubMainWindow, ListedSubWindow):
    def __init__(self, title="Frictions", study=None, parent=None):
        self._study = study
        self._reach = self._study.river.current_reach()
        self._frictions = self._reach.frictions

        self.setup_title(title)

        super(FrictionsWindow, self).__init__(
            name=self._title, ui="Frictions", parent=parent
        )

        self.setup_sc()
        self.setup_table()
        self.setup_graph()
        self.setup_connections()

        self.ui.setWindowTitle(self._title)

    def setup_title(self, title):
        self._title = (
            title + " - "
            + self._study.name + " - "
            + self._reach.name
        )

    def setup_sc(self):
        self._undo_stack = QUndoStack()

        self.undo_sc = QShortcut(QKeySequence.Undo, self)
        self.redo_sc = QShortcut(QKeySequence.Redo, self)
        self.copy_sc = QShortcut(QKeySequence.Copy, self)
        self.paste_sc = QShortcut(QKeySequence.Paste, self)

    def setup_table(self):
        self._table = {}

        table = self.find(QTableView, f"tableView")
        self._table = TableModel(
            data = self._reach,
            study = self._study,
            undo = self._undo_stack,
        )
        table.setModel(self._table)

        self._delegate_stricklers = ComboBoxDelegate(
            data = self._reach,
            study = self._study,
            mode = "stricklers",
            parent=self
        )

        table.setItemDelegateForColumn(
            3, self._delegate_stricklers
        )
        table.setItemDelegateForColumn(
            4, self._delegate_stricklers
        )

        table.setSelectionBehavior(QAbstractItemView.SelectRows)
        table.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
        table.setAlternatingRowColors(True)

    def setup_graph(self):
        self.canvas = MplCanvas(width=5, height=4, dpi=100)
        self.canvas.setObjectName("canvas")
        self.plot_layout = self.find(QVBoxLayout, "verticalLayout")
        self.plot_layout.addWidget(self.canvas)

        self.plot = PlotKPZ(
            canvas = self.canvas,
            data = self._reach.reach,
            toolbar = None,
            display_current = False
        )
        self.plot.draw()

        self.canvas_2 = MplCanvas(width=5, height=4, dpi=100)
        self.canvas_2.setObjectName("canvas_2")
        self.plot_layout_2 = self.find(QVBoxLayout, "verticalLayout_2")
        self.plot_layout_2.addWidget(self.canvas_2)

        self.plot_2 = PlotStricklers(
            canvas = self.canvas_2,
            data = self._reach,
            toolbar = None
        )
        self.plot_2.draw()


    def setup_connections(self):
        self.find(QAction, "action_add").triggered.connect(self.add)
        self.find(QAction, "action_del").triggered.connect(self.delete)
        self.find(QAction, "action_sort").triggered.connect(self.sort)
        self.find(QAction, "action_edit_stricklers").triggered.connect(self.edit_stricklers)

        self.undo_sc.activated.connect(self.undo)
        self.redo_sc.activated.connect(self.redo)
        self.copy_sc.activated.connect(self.copy)
        self.paste_sc.activated.connect(self.paste)

        table = self.find(QTableView, f"tableView")
        table.selectionModel()\
             .selectionChanged\
             .connect(self._set_current_reach)

        self._table.dataChanged\
                   .connect(self._set_current_reach)

    def index_selected_rows(self):
        table = self.find(QTableView, f"tableView")
        return list(
            # Delete duplicate
            set(
                map(
                    lambda i: i.row(),
                    table.selectedIndexes()
                )
            )
        )

    def _set_current_reach(self):
        rows = self.index_selected_rows()

        data = None
        reach = None
        highlight = None

        if len(rows) > 0:
            data = self._reach
            reach = self._reach.reach
            sec = self._frictions.get(rows[0])
            highlight = (sec.begin_kp, sec.end_kp)

        self.plot = PlotKPZ(
            canvas = self.canvas,
            data = reach,
            toolbar = None,
            display_current = False
        )
        self.plot.draw(highlight=highlight)

        self.plot = PlotStricklers(
            canvas = self.canvas_2,
            data = data,
            toolbar = None
        )
        self.plot.draw(highlight=highlight)

    def add(self):
        rows = self.index_selected_rows()
        if len(self._frictions) == 0 or len(rows) == 0:
            self._table.add(0)
        else:
            self._table.add(rows[0])

    def delete(self):
        rows = self.index_selected_rows()
        if len(rows) == 0:
            return

        self._table.delete(rows)

    def sort(self):
        self._table.sort(False)

    def move_up(self):
        row = self.index_selected_row()
        self._table.move_up(row)

    def move_down(self):
        row = self.index_selected_row()
        self._table.move_down(row)

    def copy(self):
        logger.info("TODO: copy")

    def paste(self):
        logger.info("TODO: paste")

    def undo(self):
        self._table.undo()

    def redo(self):
        self._table.redo()

    def edit_stricklers(self):
        self.strick = StricklersWindow(
            study = self._study,
            config = self.parent.conf,
            parent = self
        )
        self.strick.show()
