# -*- coding: utf-8 -*-

from tools import timer, trace

from View.ASubWindow import ASubMainWindow
from View.ListedSubWindow import ListedSubWindow

from PyQt5.QtGui import (
    QKeySequence,
)

from PyQt5.QtCore import (
    Qt, QVariant, QAbstractTableModel, QCoreApplication,
)

from PyQt5.QtWidgets import (
    QDialogButtonBox, QPushButton, QLineEdit,
    QFileDialog, QTableView, QAbstractItemView,
    QUndoStack, QShortcut, QAction, QItemDelegate,
    QHeaderView,
)

from View.Plot.MplCanvas import MplCanvas

from View.LateralContribution.translate import long_types
from View.LateralContribution.Edit.Table import TableModel, ExTimeDelegate
from View.LateralContribution.Edit.Plot import Plot

_translate = QCoreApplication.translate

class EditLateralContributionWindow(ASubMainWindow, ListedSubWindow):
    def __init__(self, title="Edit lateral conditribution",
                 data=None, study=None, parent=None):
        self._data = data
        self._study = study
        self._title = title

        self.compute_title()

        super(EditLateralContributionWindow, self).__init__(
            name=self._title, ui="EditLateralContribution", parent=parent
        )

        self.ui.setWindowTitle(self._title)

        self.setup_sc()
        self.setup_table()
        self.setup_plot()
        self.setup_connections()

    def compute_title(self):
        if self._data is not None:
            edge_name = (self._data.edge.name if self._data.edge is not None
                         else _translate("LateralContribution", "Not associate"))
            self._title = (
                _translate("Edit Lateral contribution", self._title) +
                f" - {self._study.name} " +
                f" - {self._data.name} " +
                f"({long_types[self._data.lctype]} - {edge_name})"
            )

    def setup_sc(self):
        self._undo_stack = QUndoStack()

        self.undo_sc = QShortcut(QKeySequence.Undo, self)
        self.redo_sc = QShortcut(QKeySequence.Redo, self)
        self.copy_sc = QShortcut(QKeySequence.Copy, self)
        self.paste_sc = QShortcut(QKeySequence.Paste, self)

    def setup_table(self):
        table = self.find(QTableView, "tableView")
        self._table = TableModel(
            data = self._data,
            undo = self._undo_stack,
            mode = self._study.time_system
        )

        if self._data.header[0] == "time":
            self._delegate_time = ExTimeDelegate(
                data = self._data,
                mode = self._study.time_system,
                parent = self
            )

            table.setItemDelegateForColumn(
                0, self._delegate_time
            )

        table.setModel(self._table)
        table.setSelectionBehavior(QAbstractItemView.SelectRows)
        table.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
        table.setAlternatingRowColors(True)

    def setup_plot(self):
        self.canvas = MplCanvas(width=5, height=4, dpi=100)
        self.canvas.setObjectName("canvas")
        self.verticalLayout.addWidget(self.canvas)

        self.plot = Plot(
            canvas = self.canvas,
            data = self._data,
            mode = self._study.time_system,
        )
        self.plot.draw()


    def setup_connections(self):
        self.find(QAction, "action_add").triggered.connect(self.add)
        self.find(QAction, "action_del").triggered.connect(self.delete)
        self.find(QAction, "action_sort").triggered.connect(self.sort)

        self.undo_sc.activated.connect(self.undo)
        self.redo_sc.activated.connect(self.redo)
        self.copy_sc.activated.connect(self.copy)
        self.paste_sc.activated.connect(self.paste)

        self._table.dataChanged.connect(self.update)

    def update(self):
        self.plot.update()

    def index_selected_row(self):
        table = self.find(QTableView, "tableView")
        return table.selectionModel()\
                    .selectedRows()[0]\
                    .row()

    def index_selected_rows(self):
        table = self.find(QTableView, "tableView")
        return list(
            # Delete duplicate
            set(
                map(
                    lambda i: i.row(),
                    table.selectedIndexes()
                )
            )
        )


    def add(self):
        rows = self.index_selected_rows()
        if len(self._data) == 0 or len(rows) == 0:
            self._table.add(0)
        else:
            self._table.add(rows[0])

        self.plot.update()

    def delete(self):
        rows = self.index_selected_rows()
        if len(rows) == 0:
            return

        self._table.delete(rows)
        self.plot.update()

    def sort(self):
        self._table.sort(False)
        self.plot.update()

    def move_up(self):
        row = self.index_selected_row()
        self._table.move_up(row)
        self.plot.update()

    def move_down(self):
        row = self.index_selected_row()
        self._table.move_down(row)
        self.plot.update()


    def copy(self):
        rows = self.index_selected_rows()

        table = []
        table.append(self._data.header)

        data = self._data.data
        for row in rows:
            table.append(list(data[row]))

        self.copyTableIntoClipboard(table)

    def paste(self):
        header, data = self.parseClipboardTable()

        if len(data) == 0:
            return

        row = 0
        rows = self.index_selected_rows()
        if len(rows) != 0:
            row = rows[0]

        self._table.paste(row, header, data)
        self.plot.update()

    def undo(self):
        self._table.undo()
        self.plot.update()

    def redo(self):
        self._table.redo()
        self.plot.update()
