# -*- coding: utf-8 -*-

from tools import trace, timer

from PyQt5.QtCore import (
    Qt, QVariant, QAbstractTableModel,
    QCoreApplication, QModelIndex, pyqtSlot,
    QRect,
)

from PyQt5.QtWidgets import (
    QDialogButtonBox, QPushButton, QLineEdit,
    QFileDialog, QTableView, QAbstractItemView,
    QUndoStack, QShortcut, QAction, QItemDelegate,
    QComboBox,
)

from View.LateralContribution.UndoCommand import (
    SetNameCommand, SetEdgeCommand, SetTypeCommand,
    SetBeginCommand, SetEndCommand,
    AddCommand, DelCommand, SortCommand,
    MoveCommand, PasteCommand, DuplicateCommand,
)

from Model.LateralContribution.LateralContributionTypes import (
    NotDefined, LateralContrib, Rain, Evaporation,
)
from View.LateralContribution.translate import *

_translate = QCoreApplication.translate

class ComboBoxDelegate(QItemDelegate):
    def __init__(self, data=None, mode="type", tab="", parent=None):
        super(ComboBoxDelegate, self).__init__(parent)

        self._data = data
        self._mode = mode
        self._tab = tab

    def createEditor(self, parent, option, index):
        self.editor = QComboBox(parent)

        if self._mode == "type":
            lst = list(
                map(
                    lambda k: long_types[k],
                    filter(
                        lambda k: self._tab in LC_types[k].compatibility(),
                        LC_types.keys()
                    )
                )
            )
            self.editor.addItems(lst)
        else:
            self.editor.addItems(
                [_translate("LateralContribution", "Not associate")] +
                self._data.edges_names()
            )

        self.editor.setCurrentText(index.data(Qt.DisplayRole))
        return self.editor

    def setEditorData(self, editor, index):
        value = index.data(Qt.DisplayRole)
        self.editor.currentTextChanged.connect(self.currentItemChanged)

    def setModelData(self, editor, model, index):
        text = str(editor.currentText())
        model.setData(index, text)
        editor.close()
        editor.deleteLater()

    def updateEditorGeometry(self, editor, option, index):
        r = QRect(option.rect)
        if self.editor.windowFlags() & Qt.Popup and editor.parent() is not None:
            r.setTopLeft(self.editor.parent().mapToGlobal(r.topLeft()))
        editor.setGeometry(r)

    @pyqtSlot()
    def currentItemChanged(self):
        self.commitData.emit(self.sender())


class TableModel(QAbstractTableModel):
    def __init__(self, data=None, undo=None, tab=""):
        super(QAbstractTableModel, self).__init__()
        self._headers = list(table_headers.keys())
        self._data = data
        self._undo = undo
        self._tab = tab
        self._lcs = self._data.lateral_contribution

    def flags(self, index):
        options = Qt.ItemIsEnabled | Qt.ItemIsSelectable
        options |= Qt.ItemIsEditable

        return options

    def rowCount(self, parent):
        return self._lcs.len(self._tab)

    def columnCount(self, parent):
        return len(self._headers)

    def data(self, index, role):
        if role != Qt.ItemDataRole.DisplayRole:
            return QVariant()

        row = index.row()
        column = index.column()

        if self._headers[column] == "name":
            return self._lcs.get(self._tab, row).name
        elif self._headers[column] == "type":
            t = self._lcs.get(self._tab, row).lctype
            return long_types[t]
        elif self._headers[column] == "edge":
            n = self._lcs.get(self._tab, row).edge
            if n is None:
                return _translate("LateralContribution", "Not associate")
            return n.name
        elif self._headers[column] == "begin_kp":
            return self._lcs.get(self._tab, row).begin_kp
        elif self._headers[column] == "end_kp":
            return self._lcs.get(self._tab, row).end_kp


        return QVariant()

    def headerData(self, section, orientation, role):
        if role == Qt.ItemDataRole.DisplayRole and orientation == Qt.Orientation.Horizontal:
            return table_headers[self._headers[section]]

        return QVariant()

    def setData(self, index, value, role=Qt.EditRole):
        if not index.isValid() or role != Qt.EditRole:
            return False

        row = index.row()
        column = index.column()

        if self._headers[column] == "name":
            self._undo.push(
                SetNameCommand(
                    self._lcs, self._tab, row, value
                )
            )
        elif self._headers[column] == "type":
            key = next(k for k, v in long_types.items() if v == value)
            self._undo.push(
                SetTypeCommand(
                    self._lcs, self._tab, row, LC_types[key]
                )
            )
        elif self._headers[column] == "edge":
            self._undo.push(
                SetEdgeCommand(
                    self._lcs, self._tab, row, self._data.edge(value)
                )
            )
        elif self._headers[column] == "begin_kp":
            self._undo.push(
                SetBeginCommand(
                    self._lcs, self._tab, row, value
                )
            )
        elif self._headers[column] == "end_kp":
            self._undo.push(
                SetEndCommand(
                    self._lcs, self._tab, row, value
                )
            )

        self.dataChanged.emit(index, index)
        return True

    def add(self, row, parent=QModelIndex()):
        self.beginInsertRows(parent, row, row - 1)

        self._undo.push(
            AddCommand(
                self._lcs, self._tab, row
            )
        )

        self.endInsertRows()
        self.layoutChanged.emit()

    def delete(self, rows, parent=QModelIndex()):
        self.beginRemoveRows(parent, rows[0], rows[-1])

        self._undo.push(
            DelCommand(
                self._lcs, self._tab, rows
            )
        )

        self.endRemoveRows()
        self.layoutChanged.emit()

    def sort(self, _reverse, parent=QModelIndex()):
        self.layoutAboutToBeChanged.emit()

        self._undo.push(
            SortCommand(
                self._lcs, self._tab, False
            )
        )

        self.layoutAboutToBeChanged.emit()
        self.layoutChanged.emit()

    def move_up(self, row, parent=QModelIndex()):
        if row <= 0:
            return

        target = row + 2

        self.beginMoveRows(parent, row - 1, row - 1, parent, target)

        self._undo_stack.push(
            MoveCommand(
                self._lcs, self._tab,"up", row
            )
        )

        self.endMoveRows()
        self.layoutChanged.emit()

    def move_down(self, index, parent=QModelIndex()):
        if row > len(self._lcs):
            return

        target = row

        self.beginMoveRows(parent, row + 1, row + 1, parent, target)

        self._undo_stack.push(
            MoveCommand(
                self._lcs, self._tab,"down", row
            )
        )

        self.endMoveRows()
        self.layoutChanged.emit()

    def undo(self):
        self._undo.undo()
        self.layoutChanged.emit()

    def redo(self):
        self._undo.redo()
        self.layoutChanged.emit()
