# -*- coding: utf-8 -*-

from Model.Network.Node import Node
from Model.Network.Edge import Edge
from Model.Network.Graph import Graph
from View.ASubWindow import ASubWindow
from View.Network.GraphWidget import GraphWidget
from View.Network.UndoCommand import *

from PyQt5.QtCore import (
    Qt, QRect, QVariant, QAbstractTableModel, pyqtSlot, pyqtSignal,
    QEvent,
)

from PyQt5.QtWidgets import (
    QTableView, QItemDelegate, QComboBox, QLineEdit, QHBoxLayout, QSlider,
    QPushButton, QCheckBox, QStyledItemDelegate, QStyleOptionButton, QStyle,
    QApplication,
)


class ComboBoxDelegate(QItemDelegate):
    def __init__(self, graph=None, parent=None):
        super(ComboBoxDelegate, self).__init__(parent)
        self.graph = graph

    def createEditor(self, parent, option, index):
        self.editor = QComboBox(parent)
        self.editor.addItems(self.graph.nodes_names())
        self.editor.setCurrentText(index.data(Qt.DisplayRole))
        return self.editor

    def setEditorData(self, editor, index):
        value = index.data(Qt.DisplayRole)
        self.editor.currentTextChanged.connect(self.currentItemChanged)

    def setModelData(self, editor, model, index):
        text = str(editor.currentText())
        model.setData(index, text)
        editor.close()
        editor.deleteLater()

    def updateEditorGeometry(self, editor, option, index):
        r = QRect(option.rect)
        if self.editor.windowFlags() & Qt.Popup and editor.parent() is not None:
            r.setTopLeft(self.editor.parent().mapToGlobal(r.topLeft()))
        editor.setGeometry(r)

    @pyqtSlot()
    def currentItemChanged(self):
        self.commitData.emit(self.sender())

class TrueFalseComboBoxDelegate(QItemDelegate):
    def __init__(self, parent=None):
        super(TrueFalseComboBoxDelegate, self).__init__(parent)

    def createEditor(self, parent, option, index):
        self.editor = QComboBox(parent)
        self.editor.addItems(["true", "false"])
        self.editor.setCurrentText("true" if index.data(Qt.DisplayRole) else "false")
        return self.editor

    def setEditorData(self, editor, index):
        value = str(index.data(Qt.DisplayRole))
        self.editor.currentTextChanged.connect(self.currentItemChanged)

    def setModelData(self, editor, model, index):
        value = str(editor.currentText()) == "true"
        model.setData(index, value)
        editor.close()
        editor.deleteLater()

    def updateEditorGeometry(self, editor, option, index):
        r = QRect(option.rect)
        if self.editor.windowFlags() & Qt.Popup and editor.parent() is not None:
            r.setTopLeft(self.editor.parent().mapToGlobal(r.topLeft()))
        editor.setGeometry(r)

    @pyqtSlot()
    def currentItemChanged(self):
        self.commitData.emit(self.sender())

class GraphTableModel(QAbstractTableModel):
    def __init__(self, headers=[], graph=None, undo = None, rows_type="nodes"):
        super(QAbstractTableModel, self).__init__()
        self.headers = headers
        self.graph = graph
        self._type = rows_type
        self._undo = undo

        if self._type == "nodes":
            self.rows = graph.nodes()
        elif self._type == "edges":
            self.rows = graph.edges()

    def flags(self, index):
        options = Qt.ItemIsEnabled | Qt.ItemIsSelectable

        if self.headers[index.column()] != "type":
            options |= Qt.ItemIsEditable

        return options

    def rowCount(self, parent):
        return len(self.rows)

    def columnCount(self, parent):
        return len(self.headers)

    def data(self, index, role):
        if role != Qt.ItemDataRole.DisplayRole:
            return QVariant()

        if self.headers[index.column()] == "type":
            node = self.rows[index.row()]
            ret = "internal"

            if not self.graph.is_enable_node(node):
                ret = "disable"
            elif self.graph.is_upstream_node(node):
                ret = "upstream"
            elif self.graph.is_downstream_node(node):
                ret = "downstream"

            return ret

        return self.rows[index.row()][self.headers[index.column()]]

    def headerData(self, section, orientation, role):
        if (role == Qt.ItemDataRole.DisplayRole and
            orientation == Qt.Orientation.Horizontal):
            return self.headers[section].capitalize()

        return QVariant()

    @pyqtSlot()
    def setData(self, index, value, role=Qt.EditRole):
        if index.isValid():
            if role == Qt.EditRole:
                if (self.headers[index.column()] == "node1" or
                    self.headers[index.column()] == "node2"):
                    node = self.graph.node(value)
                    self._undo.push(
                        SetNodeCommand(
                            self.graph,
                            self.rows[index.row()],
                            self.headers[index.column()],
                            node
                        )
                    )
                elif self.headers[index.column()] == "enable":
                    self._undo.push(
                        EnableEdgeCommand(
                            self.rows[index.row()], value
                        )
                    )
                else:
                    self._undo.push(
                        SetCommand(
                            self.rows[index.row()],
                            self.headers[index.column()],
                            value
                        )
                    )

                self.dataChanged.emit(index, index, [Qt.DisplayRole])
                self.layoutChanged.emit()
                return True

            self.dataChanged.emit(index, index)
        else:
            return False

    def update(self):
        if self._type == "nodes":
            self.rows = self.graph.nodes()
        elif self._type == "edges":
            self.rows = self.graph.edges()

        self.layoutChanged.emit()

    def reverse_edge(self, index):
        if self._type == "edges":
            tmp = self.rows[index.row()].node1
            self.rows[index.row()].node1 = self.rows[index.row()].node2
            self.rows[index.row()].node2 = tmp
            self.dataChanged.emit(index, index, [Qt.DisplayRole])
            self.layoutChanged.emit()

    def undo(self):
        self._undo.undo()
        self.layoutChanged.emit()

    def redo(self):
        self._undo.redo()
        self.layoutChanged.emit()
