# -*- coding: utf-8 -*-

import logging

from tools import trace, timer

from View.ASubWindow import ASubMainWindow
from View.ListedSubWindow import ListedSubWindow

from PyQt5.QtGui import (
    QKeySequence,
)

from PyQt5.QtCore import (
    Qt, QVariant, QAbstractTableModel,
    QCoreApplication, QModelIndex, pyqtSlot,
    QRect,
)

from PyQt5.QtWidgets import (
    QDialogButtonBox, QPushButton, QLineEdit,
    QFileDialog, QTableView, QAbstractItemView,
    QUndoStack, QShortcut, QAction, QItemDelegate,
    QComboBox, QVBoxLayout, QHeaderView, QTabWidget,
)

from View.SedimentLayers.Reach.Profile.UndoCommand import *
from View.SedimentLayers.Reach.Profile.Table import *
from View.SedimentLayers.Reach.Profile.Plot import Plot

from View.Plot.MplCanvas import MplCanvas
from View.SedimentLayers.Reach.Profile.translate import *

from View.SedimentLayers.Window import SedimentLayersWindow

_translate = QCoreApplication.translate

logger = logging.getLogger()

class ProfileSedimentLayersWindow(ASubMainWindow, ListedSubWindow):
    def __init__(self, title="Profile sediment layers", study=None, profile=None, parent=None):
        self._study = study
        self._sediment_layers = self._study.river.sediment_layers
        self._profile = profile
        self._reach = self._study.river.current_reach().reach

        self.setup_title(title)

        super(ProfileSedimentLayersWindow, self).__init__(
            name=self._title, ui="ProfileSedimentLayers", parent=parent
        )

        self.setup_sc()
        self.setup_table()
        self.setup_graph()
        self.setup_connections()

        self.ui.setWindowTitle(self._title)

    def setup_title(self, title):
        rname = self._reach.name
        if rname == "":
            rname = _translate("SedimentLayers", "(no name)")

        pname = self._profile.name
        if pname == "":
            pname = _translate(
                "SedimentLayers",
                "(no name - @kp)").replace("@kp", str(self._profile.kp)
            )

        self._title = (
            title + " - "
            + self._study.name + " - "
            + rname + " - " + pname
        )

    def setup_sc(self):
        self._undo_stack = QUndoStack()

        self.undo_sc = QShortcut(QKeySequence.Undo, self)
        self.redo_sc = QShortcut(QKeySequence.Redo, self)
        self.copy_sc = QShortcut(QKeySequence.Copy, self)
        self.paste_sc = QShortcut(QKeySequence.Paste, self)

    def setup_table(self):
        table = self.find(QTableView, f"tableView")
        self._table = TableModel(
            study = self._study,
            profile = self._profile,
            undo = self._undo_stack,
        )
        table.setModel(self._table)

        self._delegate_stricklers = ComboBoxDelegate(
            study = self._study,
            parent=self
        )

        table.setItemDelegateForColumn(
            list(table_headers).index("sl"),
            self._delegate_stricklers
        )

        table.setSelectionBehavior(QAbstractItemView.SelectRows)
        table.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
        table.setAlternatingRowColors(True)

    def setup_graph(self):
        self.canvas = MplCanvas(width=5, height=4, dpi=100)
        self.canvas.setObjectName("canvas")
        self.plot_layout = self.find(QVBoxLayout, "verticalLayout")
        self.plot_layout.addWidget(self.canvas)

        self._update_plot()

    def _update_plot(self):
        self.plot = Plot(
            canvas = self.canvas,
            data = self._profile,
            toolbar = None,
            display_current = False
        )
        self.plot.draw()


    def setup_connections(self):
        self.undo_sc.activated.connect(self.undo)
        self.redo_sc.activated.connect(self.redo)
        self.copy_sc.activated.connect(self.copy)
        self.paste_sc.activated.connect(self.paste)

        self._table.layoutChanged\
                   .connect(self._update_plot)
        self._table.dataChanged\
                   .connect(self._update_plot)


    def index_selected_rows(self):
        table = self.find(QTableView, f"tableView")
        return list(
            # Delete duplicate
            set(
                map(
                    lambda i: i.row(),
                    table.selectedIndexes()
                )
            )
        )

    def copy(self):
        logger.info("TODO: copy")

    def paste(self):
        logger.info("TODO: paste")

    def undo(self):
        self._table.undo()

    def redo(self):
        self._table.redo()
